@@ -853,7 +853,8 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 		update_leafs_children_changed_size (cur, leaf_children_changed_size);
 
 		cur->x = x;
-		html_object_calc_size (cur, painter, changed_objs);
+		if (cur != begin)
+			html_object_calc_size (cur, painter, changed_objs);
 
 		valign = html_object_get_valign (cur);
 		if ((!is_top_aligned (valign) && (cur->ascent > a || cur->descent > d))
@@ -919,10 +920,10 @@ layout_aligned (HTMLObject *o, HTMLPainter *painter, HTMLObject *cur,
 }
 
 static gboolean
-layout (HTMLObject *o, HTMLPainter *painter, GList **changed_objs, gboolean *leaf_children_changed_size)
+html_clue_flow_layout (HTMLObject *o, HTMLPainter *painter, GList **changed_objs, gboolean *leaf_children_changed_size)
 {
 	HTMLClueFlow *cf = HTML_CLUEFLOW (o);
-	HTMLObject *cur = HTML_CLUE (o)->head, *end;
+	HTMLObject *cur = HTML_CLUE (o)->head;
 	gint indent, lmargin, rmargin;
 	gboolean changed = FALSE;
 
@@ -932,19 +933,18 @@ layout (HTMLObject *o, HTMLPainter *painter, GList **changed_objs, gboolean *lea
 
 	while (cur) {
 		if (cur->flags & HTML_OBJECT_FLAG_ALIGNED)
-			end = layout_aligned (o, painter, cur, changed_objs, leaf_children_changed_size,
+			cur = layout_aligned (o, painter, cur, changed_objs, leaf_children_changed_size,
 					      &lmargin, &rmargin, indent, &changed);
 		else
-			end = layout_line (o, painter, cur, changed_objs, leaf_children_changed_size,
+			cur = layout_line (o, painter, cur, changed_objs, leaf_children_changed_size,
 					   &lmargin, &rmargin, indent);
-		cur = end;
 	}
 
 	return changed;
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
+html_clue_flow_real_calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLClueFlow *cf = HTML_CLUEFLOW (o);
 	gint oa, od, ow, padding;
@@ -965,7 +965,7 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 	/* calc size */
 	padding = calc_padding (painter);
 	add_pre_padding (cf, padding);
-	changed = layout (o, painter, changed_objs, &leaf_children_changed_size);
+	changed = html_clue_flow_layout (o, painter, changed_objs, &leaf_children_changed_size);
 	add_post_padding (cf, padding);
 
 	/* take care about changes */
@@ -1736,6 +1736,34 @@ append_selection_string (HTMLObject *self,
 	}
 }
 
+static void
+forward_items (PangoItem **items, int *ii, int *io, int n)
+{
+	while (n > 0) {
+		if (items [*ii]->num_chars - *io > n) {
+			*io += n;
+			return;
+		}
+		n -= items [*ii]->num_chars - *io;
+		(*ii) ++;
+		*io = 0;
+	}
+}
+
+static void
+backward_items (PangoItem **items, int *ii, int *io, int n)
+{
+	while (n > 0) {
+		if (*io >= n) {
+			*io -= n;
+			return;
+		}
+		n -= *io;
+		(*ii) --;
+		*io = items [*ii]->num_chars - 1;
+	}
+}
+
 static gboolean
 save_plain (HTMLObject *self,
 	    HTMLEngineSaveState *state,
@@ -1773,28 +1801,30 @@ save_plain (HTMLObject *self,
 			g_string_append (out, "\n");
 		} else {
 			PangoAttrList *attrs = pango_attr_list_new ();
-			gint bytes = strlen (s), i, slen = g_utf8_strlen (s, -1), clen, n;
-			GList *items, *cur;
+			gint bytes = strlen (s), i, slen = g_utf8_strlen (s, -1), clen, n_items, cii, cio;
+			GList *items_list, *cur;
 			PangoContext *pc = gtk_widget_get_pango_context (GTK_WIDGET (state->engine->widget));
 			PangoLogAttr *lattrs;
+			PangoItem **items;
 			gint len, skip;
 
-			items = pango_itemize (pc, s, 0, bytes, attrs, NULL);
+			items_list = pango_itemize (pc, s, 0, bytes, attrs, NULL);
 			lattrs = g_new (PangoLogAttr, slen + 1);
-			n = g_list_length (items);
+			n_items = g_list_length (items_list);
+			items = g_new (PangoItem *, n_items);
 			clen = 0;
-			for (i = 0, cur = items; i < n; i ++, cur = cur->next) {
+			for (i = 0, cur = items_list; i < n_items; i ++, cur = cur->next) {
 				PangoItem *item;
 
 				item = (PangoItem *) cur->data;
 				pango_break (s + item->offset, item->length, &item->analysis, lattrs + clen, item->num_chars + 1);
 				clen += item->num_chars;
-				pango_item_free (item);
+				items [i] = item;
 			}
-			g_list_free (items);
+			g_list_free (items_list);
 			pango_attr_list_unref (attrs);
 
-			clen = 0;
+			clen = cii = cio = 0;
 			while (*s) {
 				len = strcspn (s, "\n");
 				len = g_utf8_strlen (s, len);
@@ -1805,30 +1835,41 @@ save_plain (HTMLObject *self,
 					if (len > max_len) {
 						gint l = max_len;
 						gboolean look_backward = TRUE;
-						gint wi, wl;
+						gint wi, wl, lii, lio, blii, blio;
 
-						wi = wl = clen + max_len;
+						wl = clen + max_len;
+						lii = cii;
+						lio = cio;
+						forward_items (items, &lii, &lio, max_len);
+						blii = lii;
+						blio = lio;
 
 						if (lattrs [wl].is_white) {
 
-							while (lattrs [wl].is_white && wl < slen)
+							while (lattrs [wl].is_white && wl < slen) {
 								wl ++;
-							if (wl < slen && lattrs [wl].is_line_break)
+								forward_items (items, &lii, &lio, 1);
+							}
+							if (wl < slen && html_text_is_line_break (items [lii], lattrs, wl))
 								look_backward = FALSE;
-							else
+							else {
 								wl = clen + max_len;
+								lii = blii;
+								lio = blio;
+							}
 
 						}
 
 						if (look_backward) {
 							while (wl > 0) {
-								if (lattrs [wl].is_line_break)
+								if (html_text_is_line_break (items [lii], lattrs, wl))
 									break;
 								wl --;
+								backward_items (items, &lii, &lio, 1);
 							}
 						}
 
-						if (wl > clen && wl < slen && lattrs [wl].is_line_break) {
+						if (wl > clen && wl < slen && html_text_is_line_break (items [lii], lattrs, wl)) {
 							wi = MIN (wl, clen + max_len);
 							while (wi > 0 && lattrs [wi - 1].is_white)
 								wi --;
@@ -1865,16 +1906,21 @@ save_plain (HTMLObject *self,
 				s += bytes;
 				s = g_utf8_offset_to_pointer (s, skip);
 				clen += len + skip;
+				forward_items (items, &cii, &cio, len + skip);
 
 				if (*s == '\n') {
 					s++;
 					clen ++;
+					forward_items (items, &cii, &cio, 1);
 				}
 			
 				g_string_append_c (out, '\n');
 				firstline = FALSE;
 			}
 			g_free (lattrs);
+			for (i = 0; i < n_items; i ++)
+				pango_item_free (items [i]);
+			g_free (items);
 		}
 
 		if (get_post_padding (flow, calc_padding (state->engine->painter)) > 0) {
@@ -2215,7 +2261,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->op_copy = op_copy;
 	object_class->split = split;
 	object_class->merge = merge;
-	object_class->calc_size = calc_size;
+	object_class->calc_size = html_clue_flow_real_calc_size;
 	object_class->set_max_width = set_max_width;
 	object_class->set_max_height = set_max_height;
 	object_class->calc_min_width = calc_min_width;
