@@ -256,7 +256,7 @@ cut_attr_list (HTMLText *text, gint begin_index, gint end_index)
 }
 
 static void
-cut_links (HTMLText *text, gint start_offset, gint end_offset)
+cut_links (HTMLText *text, gint start_offset, gint end_offset, gint start_index, gint end_index)
 {
 	GSList *l, *next;
 	Link *link;
@@ -268,10 +268,23 @@ cut_links (HTMLText *text, gint start_offset, gint end_offset)
 		if (start_offset <= link->start_offset && link->end_offset <= end_offset) {
 			html_link_free (link);
 			text->links = g_slist_delete_link (text->links, l);
-		} else if (start_offset <= link->start_offset && link->start_offset <= end_offset)
-			link->start_offset = end_offset;
-		else if (start_offset <= link->end_offset && link->end_offset <= end_offset)
+		} else if (end_offset <= link->start_offset) {
+			link->start_offset -= (end_offset - start_offset);
+			link->start_index -= (end_index - start_index);
+			link->end_offset -= (end_offset - start_offset);
+			link->end_index -= (end_index - start_index);
+		} else if (start_offset <= link->start_offset)  {
+			link->start_offset = start_offset;
+			link->end_offset -= (end_offset - start_offset);
+			link->start_index = start_index;
+			link->end_index -= (end_index - start_index);
+		} else if (end_offset <= link->end_offset) {
+			link->end_offset -= (end_offset - start_offset);
+			link->end_index -= (end_index - start_index);
+		} else if (start_offset <= link->end_offset) {
 			link->end_offset = start_offset;
+			link->end_index = start_index;
+		}
 	}
 }
 
@@ -294,21 +307,20 @@ html_text_op_copy_helper (HTMLText *text, GList *from, GList *to, guint *len)
 
 	rv = html_object_dup (HTML_OBJECT (text));
 	rvt = HTML_TEXT (rv);
-
-	if (end_index < rvt->text_bytes)
-		cut_attr_list (rvt, end_index, rvt->text_bytes);
-	if (begin_index > 0)
-		cut_attr_list (rvt, 0, begin_index);
-	if (end < rvt->text_len)
-		cut_links (rvt, end, rvt->text_len);
-	if (begin > 0)
-		cut_links (rvt, 0, begin);
-
-	nt = g_strndup (rvt->text + begin_index, end_index - begin_index);
-	g_free (rvt->text);
-	rvt->text = nt;
 	rvt->text_len = end - begin;
 	rvt->text_bytes = end_index - begin_index;
+	nt = g_strndup (rvt->text + begin_index, rvt->text_bytes);
+	g_free (rvt->text);
+	rvt->text = nt;
+
+	if (end_index < text->text_bytes)
+		cut_attr_list (rvt, end_index, text->text_bytes);
+	if (begin_index > 0)
+		cut_attr_list (rvt, 0, begin_index);
+	if (end < text->text_len)
+		cut_links (rvt, end, text->text_len, end_index, text->text_bytes);
+	if (begin > 0)
+		cut_links (rvt, 0, begin, 0, begin_index);
 
 	return rv;
 }
@@ -350,11 +362,11 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 			cut_attr_list (rvt, end_index, rvt->text_bytes);
 		if (begin_index > 0)
 			cut_attr_list (rvt, 0, begin_index);
-		cut_links (text, begin, end);
+		cut_links (text, begin, end, begin_index, end_index);
 		if (end < rvt->text_len)
-			cut_links (rvt, end, rvt->text_len);
+			cut_links (rvt, end, rvt->text_len, end_index, rvt->text_bytes);
 		if (begin > 0)
-			cut_links (rvt, 0, begin);
+			cut_links (rvt, 0, begin, 0, begin_index);
 		nt = g_strconcat (text->text, tail, NULL);
 		g_free (text->text);
 
@@ -376,6 +388,7 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 		move_spell_errors (text->spell_errors, end, - (end - begin));
 
 		html_text_convert_nbsp (text, TRUE);
+		html_text_convert_nbsp (rvt, TRUE);
 		pango_info_destroy (text);
 	} else {
 		text->spell_errors = remove_spell_errors (text->spell_errors, 0, text->text_len);
@@ -420,7 +433,9 @@ merge_links (HTMLText *t1, HTMLText *t2)
 			Link *link = (Link *) l->data;
 
 			link->start_offset += t1->text_len;
+			link->start_index += t1->text_bytes;
 			link->end_offset += t1->text_len;
+			link->end_index += t1->text_bytes;
 		}
 
 		if (t1->links) {
@@ -429,6 +444,7 @@ merge_links (HTMLText *t1, HTMLText *t2)
 
 			if (tail->start_offset == head->end_offset && html_link_equal (head, tail)) {
 				tail->start_offset = head->start_offset;
+				tail->start_index = head->start_index;
 				html_link_free (head);
 				t1->links = g_slist_delete_link (t1->links, t1->links);
 			}
@@ -548,7 +564,7 @@ split_attrs (HTMLText *t1, HTMLText *t2, gint index)
 }
 
 static void
-split_links (HTMLText *t1, HTMLText *t2, gint offset)
+split_links (HTMLText *t1, HTMLText *t2, gint offset, gint index)
 {
 	GSList *l, *prev = NULL;
 
@@ -556,8 +572,10 @@ split_links (HTMLText *t1, HTMLText *t2, gint offset)
 		Link *link = (Link *) l->data;
 
 		if (link->start_offset < offset) {
-			if (link->end_offset > offset)
+			if (link->end_offset > offset) {
 				link->end_offset = offset;
+				link->end_index = index;
+			}
 
 			if (prev) {
 				prev->next = NULL;
@@ -582,6 +600,7 @@ split_links (HTMLText *t1, HTMLText *t2, gint offset)
 		if (link->start_offset < offset) {
 			if (link->end_offset > offset) {
 				link->start_offset = offset;
+				link->start_index = index;
 				prev = l;
 				l = l->next;
 			}
@@ -601,7 +620,9 @@ split_links (HTMLText *t1, HTMLText *t2, gint offset)
 		Link *link = (Link *) l->data;
 
 		link->start_offset -= offset;
+		link->start_index -= index;
 		link->end_offset -= offset;
+		link->end_index -= index;
 	}
 }
 
@@ -633,7 +654,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	t2->text_len   -= offset;
 	t2->text_bytes -= split_index;
 	split_attrs (t1, t2, split_index);
-	split_links (t1, t2, offset);
+	split_links (t1, t2, offset, split_index);
 	if (!html_text_convert_nbsp (t2, FALSE))
 		t2->text = g_strdup (t2->text);
 	g_free (tt);
@@ -678,7 +699,7 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 }
 
 static gboolean
-calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
+html_text_real_calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
 	self->width = 0;
 	html_object_calc_preferred_width (self, painter);
@@ -1077,16 +1098,14 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 		PangoAttrList *attrs;
 		PangoAttribute *attr;
 		gchar *translated, *heap = NULL;
-		guint bytes;
 		gint i;
 
-		bytes = strlen (text->text);
-		if (bytes > HTML_ALLOCA_MAX)
-			heap = translated = g_malloc (bytes);
+		if (text->text_bytes > HTML_ALLOCA_MAX)
+			heap = translated = g_malloc (text->text_bytes);
 		else 
-			translated = alloca (bytes);
+			translated = alloca (text->text_bytes);
 
-		html_replace_tabs (text->text, translated, bytes);
+		html_replace_tabs (text->text, translated, text->text_bytes);
 		if (HTML_IS_PLAIN_PAINTER (painter)) {
 			attrs = pango_attr_list_new ();
 			attr = pango_attr_family_new (painter->font_manager.fixed.face);
@@ -1126,7 +1145,7 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 			pango_attr_list_change (attrs, attr);
 		}
 
-		items = pango_itemize (pc, translated, 0, bytes, attrs, NULL);
+		items = pango_itemize (pc, translated, 0, text->text_bytes, attrs, NULL);
 		pango_attr_list_unref (attrs);
 
 		text->pi = html_text_pango_info_new (g_list_length (items));
@@ -1192,8 +1211,8 @@ html_text_tail_white_space (HTMLText *text, HTMLPainter *painter, gint offset, g
 		s = g_utf8_prev_char (s);
 		if (pi->entries [ii].attrs [io].is_white) {
 			if (HTML_IS_GDK_PAINTER (painter) || HTML_IS_PLAIN_PAINTER (painter)) {
-				if (*s == '\t') {
-					gint skip = 8, co = offset;
+				if (*s == '\t' && offset > 1) {
+					gint skip = 8, co = offset - 1;
 
 					do {
 						s = g_utf8_prev_char (s);
@@ -1239,6 +1258,13 @@ update_mw (HTMLText *text, HTMLPainter *painter, gint offset, gint *last_offset,
 	*last_offset = offset;
 }
 
+gboolean
+html_text_is_line_break (PangoItem *item, PangoLogAttr *attrs, int offset)
+{
+	return (attrs [offset].is_line_break
+		&& (item->analysis.lang_engine != NULL || (offset > 0 && attrs [offset - 1].is_white)));
+}
+
 static gint
 calc_min_width (HTMLObject *self, HTMLPainter *painter)
 {
@@ -1257,7 +1283,7 @@ calc_min_width (HTMLObject *self, HTMLPainter *painter)
 	while (offset < text->text_len) {
 		gint skip;
 
-		if (offset > 0 && pi->entries [ii].attrs [io].is_line_break)
+		if (offset > 0 && html_text_is_line_break (pi->entries [ii].item, pi->entries [ii].attrs, io))
 			update_mw (text, painter, offset, &last_offset, &ww, &mw, ii, io, s, line_offset);
 
 		if (*s == '\t') {
@@ -1384,6 +1410,12 @@ save_close_attrs (HTMLEngineSaveState *state, GSList *attrs)
 			tag = "</S>";
 			break;
 		case PANGO_ATTR_SIZE:
+			if (attr->klass == &html_pango_attr_font_size_klass) {
+				HTMLPangoAttrFontSize *size = (HTMLPangoAttrFontSize *) attr;
+				if ((size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3)
+					tag = "</FONT>";
+			}
+			break;
 		case PANGO_ATTR_FOREGROUND:
 			tag = "</FONT>";
 			break;
@@ -1494,7 +1526,7 @@ save (HTMLObject *self, HTMLEngineSaveState *state)
 
 			attrs = pango_attr_iterator_get_attrs (iter);
 			pango_attr_iterator_range (iter, &start_index, &end_index);
-			if (end_index == G_MAXINT)
+			if (end_index > text->text_bytes)
 				end_index = text->text_bytes;
 
 			if (l && !link_started) {
@@ -1552,52 +1584,79 @@ get_length (HTMLObject *self)
 
 /* #define DEBUG_NBSP */
 
+struct TmpDeltaRecord
+{
+	int index;
+	int delta;
+};
+
 static gboolean
-check_last_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
+check_last_white (gint white_space, gunichar last_white, gint *delta_out, gboolean *rv_out)
 {
 	if (white_space > 0 && last_white == ENTITY_NBSP) {
 		(*delta_out) --;
-		rv = TRUE;
+		*rv_out = TRUE;
+
+		return TRUE;
 	}
 
-	return rv;
+	return FALSE;
 }
 
 static gboolean
-check_prev_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
+check_prev_white (gint white_space, gunichar last_white, gint *delta_out, gboolean *rv_out)
 {
 	if (white_space > 0 && last_white == ' ') {
 		(*delta_out) ++;
-		rv = TRUE;
+		*rv_out = TRUE;
+
+		return TRUE;
 	}
 
-	return rv;
+	return FALSE;
+}
+
+static GSList *
+add_change (GSList *list, int index, int delta)
+{
+	struct TmpDeltaRecord *rec = g_new (struct TmpDeltaRecord, 1);
+
+	rec->index = index;
+	rec->delta = delta;
+
+	return g_slist_prepend (list, rec);
 }
 
 static gboolean
-is_convert_nbsp_needed (const gchar *s, gint *delta_out)
+is_convert_nbsp_needed (const gchar *s, gint *delta_out, GSList **changes_out)
 {
 	gunichar uc, last_white = 0;
-	gboolean rv = FALSE;
+	gboolean rv = FALSE, change;
 	gint white_space;
-	const gchar *p, *op;
+	const gchar *p, *op, *bop;
 
 	*delta_out = 0;
 
-	op = p = s;
+	bop = p = s;
 	white_space = 0;
-	while (*p && (uc = g_utf8_get_char (p)) && (p = g_utf8_next_char (p))) {
+	while (*p && (uc = g_utf8_get_char (p)) && (op = p) && (p = g_utf8_next_char (p))) {
 		if (uc == ENTITY_NBSP || uc == ' ') {
-			rv = check_prev_white (rv, white_space, last_white, delta_out);
+			change = check_prev_white (white_space, last_white, delta_out, &rv);
 			white_space ++;
 			last_white = uc;
 		} else {
-			rv = check_last_white (rv, white_space, last_white, delta_out);
+			change = check_last_white (white_space, last_white, delta_out, &rv);
 			white_space = 0;
 		}
-		op = p;
+		if (change)
+			*changes_out = add_change (*changes_out, bop - s, *delta_out);
+		bop = op;
 	}
-	rv = check_last_white (rv, white_space, last_white, delta_out);
+
+	if (check_last_white (white_space, last_white, delta_out, &rv))
+		*changes_out = add_change (*changes_out, op - s, *delta_out);
+
+	*changes_out = g_slist_reverse (*changes_out);
 
 	return rv;
 }
@@ -1605,13 +1664,13 @@ is_convert_nbsp_needed (const gchar *s, gint *delta_out)
 static void
 write_prev_white_space (gint white_space, gchar **fill)
 {
-			if (white_space > 0) {
+	if (white_space > 0) {
 #ifdef DEBUG_NBSP
-				printf ("&nbsp;");
+		printf ("&nbsp;");
 #endif
-				**fill = 0xc2; (*fill) ++;
-				**fill = 0xa0; (*fill) ++;
-			}
+		**fill = 0xc2; (*fill) ++;
+		**fill = 0xa0; (*fill) ++;
+	}
 }
 
 static void
@@ -1625,17 +1684,18 @@ write_last_white_space (gint white_space, gchar **fill)
 	}
 }
 
-static void
-convert_nbsp (gchar *fill, const gchar *p)
+static GSList *
+convert_nbsp (gchar *fill, const gchar *text)
 {
+	GSList *changes = NULL; 
 	gint white_space;
 	gunichar uc;
-	const gchar *op;
+	const gchar *op, *p;
 
 #ifdef DEBUG_NBSP
 	printf ("convert_nbsp: %s --> \"", p);
 #endif
-	op = p;
+	op = p = text;
 	white_space = 0;
 
 	while (*p && (uc = g_utf8_get_char (p)) && (p = g_utf8_next_char (p))) {
@@ -1662,18 +1722,106 @@ convert_nbsp (gchar *fill, const gchar *p)
 #endif
 }
 
+static void
+update_index_interval (int *start_index, int *end_index, GSList *changes)
+{
+	GSList *c;
+	int index, delta;
+
+	index = delta = 0;
+
+	for (c = changes; c && *start_index > index; c = c->next) {
+		struct TmpDeltaRecord *rec = c->data;
+
+		if (*start_index > index && *start_index <= rec->index) {
+			(*start_index) += delta;
+			break;
+		}
+		index = rec->index;
+		delta = rec->delta;
+	}
+
+	if (c == NULL && *start_index > index) {
+		(*start_index) += delta;
+		(*end_index) += delta;
+		return;
+	}
+
+	for (; c && *end_index > index; c = c->next) {
+		struct TmpDeltaRecord *rec = c->data;
+
+		if (*end_index > index && *end_index <= rec->index) {
+			(*start_index) += delta;
+			break;
+		}
+		index = rec->index;
+		delta = rec->delta;
+	}
+
+	if (c == NULL && *end_index > index)
+		(*end_index) += delta;
+}
+
+static gboolean
+update_attributes_filter (PangoAttribute *attr, gpointer data)
+{
+	update_index_interval (&attr->start_index, &attr->end_index, (GSList *) data);
+
+	return FALSE;
+}
+
+static void
+update_attributes (PangoAttrList *attrs, GSList *changes)
+{
+	pango_attr_list_filter (attrs, update_attributes_filter, changes);
+}
+
+static void
+update_links (GSList *links, GSList *changes)
+{
+	GSList *cl;
+
+	for (cl = links; cl; cl = cl->next) {
+		Link *link = (Link *) cl->data;
+		update_index_interval (&link->start_index, &link->end_index, changes);
+	}
+}
+
+static void
+free_changes (GSList *changes)
+{
+	GSList *c;
+
+	for (c = changes; c; c = c->next)
+		g_free (c->data);
+	g_slist_free (changes);
+}
+
 gboolean
 html_text_convert_nbsp (HTMLText *text, gboolean free_text)
 {
+	GSList *changes = NULL;
 	gint delta;
-	gchar *to_free;
 
-	if (is_convert_nbsp_needed (text->text, &delta)) {
-		to_free    = text->text;
+	if (is_convert_nbsp_needed (text->text, &delta, &changes)) {
+		GSList *attrs, *extra_attrs;
+		gchar *to_free;
+
+		to_free = text->text;
 		text->text = g_malloc (strlen (to_free) + delta + 1);
+		text->text_bytes += delta;
 		convert_nbsp (text->text, to_free);
 		if (free_text)
 			g_free (to_free);
+		if (changes) {
+			if (text->attr_list)
+				update_attributes (text->attr_list, changes);
+			if (text->extra_attr_list)
+				update_attributes (text->extra_attr_list, changes);
+			if (text->links)
+				update_links (text->links, changes);
+			free_changes (changes);
+		}
 		html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
 		return TRUE;
 	}
@@ -2067,7 +2215,7 @@ html_text_class_init (HTMLTextClass *klass,
 	object_class->split = object_split;
 	object_class->draw = draw;
 	object_class->accepts_cursor = accepts_cursor;
-	object_class->calc_size = calc_size;
+	object_class->calc_size = html_text_real_calc_size;
 	object_class->calc_preferred_width = calc_preferred_width;
 	object_class->calc_min_width = calc_min_width;
 	object_class->fit_line = ht_fit_line;
@@ -2503,10 +2651,13 @@ html_text_add_link_full (HTMLText *text, HTMLEngine *e, gchar *url, gchar *targe
 				html_link_free (link);
 				continue;
 			}
-			if (link->start_offset < end_offset && link->end_offset > end_offset)
+			if (link->start_offset < end_offset && link->end_offset > end_offset) {
 				link->start_offset = end_offset;
-			else if (link->end_offset > start_offset && link->end_offset <= end_offset)
+				link->start_index = end_index;
+			} else if (link->end_offset > start_offset && link->end_offset <= end_offset) {
 				link->end_offset = start_offset;
+				link->end_index = start_index;
+			}
 
 			if (link->end_offset <= start_offset) {
 				if (lprev)
@@ -2926,7 +3077,7 @@ html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine
 	}
 
 	/* size */
-	if (style & GTK_HTML_FONT_STYLE_SIZE_MASK) {
+	if ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) && ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3)) {
 		attr = html_pango_attr_font_size_new (style);
 		html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
 		attr->start_index = start_index;
